/**************************************************************************
***    
*** Copyright (c) 1995-2000 Regents of the University of California,
***               Andrew E. Caldwell, Andrew B. Kahng and Igor L. Markov
***
***  Contact author(s): abk@cs.ucsd.edu, imarkov@umich.edu
***  Original Affiliation:   UCLA, Computer Science Department,
***                          Los Angeles, CA 90095-1596 USA
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/










//created by Saurabh Adya, Dec 3 2000
#ifdef _MSC_VER
#pragma warning(disable:4786)
#endif

#include <algorithm>
#include <ABKCommon/sgi_hash_map.h>
#include "ABKCommon/abkcommon.h"
#include "RBPlacement.h"

#ifdef _MSC_VER
#ifndef rint
#define rint(a) floor((a)+0.5)
#endif
#endif

//added by sadya to remove overlaps in each subrow
void RBPlacement::remOverlaps(void)
{
  if (_params.verb.forActions) cout << " Running overlap removal ..." << endl;
  double initHPWL=evalHPWL();
  if (_params.verb.forMajStats)
                   cout << "  HPWL before overlap remover is "<<initHPWL<<endl;
  Timer tm;

  itRBPCoreRow itc;
  itRBPSubRow its;
  itRBPCellIds itn;
  itRBPCellIds itntemp;
  itRBPCellIds itnleft;
  itRBPCellIds itnright;
  itRBPCoreRow itcNext;   //used to scan immediate next row
  itRBPSubRow itsNext;    //used to scan immediate next sub row 
  itRBPCellIds itnNext;   //used to scan immediate next sub row cells


  bool leftRight=false;   //false means move left else move right
  double WS=0;
  unsigned numOverlaps=0;
  unsigned totalOverlaps=0;
  char plotFile[20];
  double rowHeight=0;
  double nodeHeight=0;
  double xDiff=0;
  //for each core row, for each subrow scan left to right if overlap found then take action
  for(itc=_coreRows.begin();itc!=_coreRows.end();++itc)
    {
      rowHeight=itc->getHeight();
      for(its=itc->subRowsBegin();its!=itc->subRowsEnd();++its)
	{
	  for(itn=its->cellIdsBegin();itn!=its->cellIdsEnd();++itn)
	    {
	      nodeHeight=_hgWDims->getNodeHeight(*itn);
	      //check for overlaps in the same row
	      xDiff=1;
	      if(itn!=its->cellIdsEnd()-1)
		{
		  itntemp=itn+1;
		  if(_placement[*itntemp].x < 
		     _placement[*itn].x+_hgWDims->getNodeWidth(*itn))
		    {
		      //overlap found, now deal with it
		      numOverlaps++;
		      totalOverlaps++;

		      //first plot the overlap area
		      sprintf(plotFile,"%s%d","overlaps",totalOverlaps);
/*		      double xMin=_placement[*itn].x -
			5*_hgWDims->getNodeWidth(*itn);
		      double xMax=_placement[*itn].x +
			5*_hgWDims->getNodeWidth(*itn);
		      double yMin=_placement[*itn].y -
			2*_hgWDims->getNodeWidth(*itn);
		      double yMax=_placement[*itn].y +
			2*_hgWDims->getNodeWidth(*itn);
*/		      //to plot the overlaps uncomment out the folowing line
		      //saveAsPlot("plotNodesWNames",xMin,xMax,yMin,yMax,plotFile);

		      xDiff= _placement[*itn].x + 
			_hgWDims->getNodeWidth(*itn)-_placement[*itntemp].x;

		      itnleft=itn;
		      itnright=itn;
		      leftRight=false;

		      while(xDiff>0 && (itnleft!=(its->cellIdsBegin()) || 
					itnright!=(its->cellIdsEnd()-2)))
			{
			  // chose the direction of search
			  if(leftRight==false && itnright!=its->cellIdsEnd()-2)
			    leftRight=true;
			  else if(leftRight==true && 
				  itnleft!=its->cellIdsBegin())
			    leftRight=false;

			  if(leftRight==true)//search right for WS
			    {
			      ++itnright;
			      itntemp=itnright+1;
			      WS=_placement[*itntemp].x -
				_placement[*itnright].x -
				_hgWDims->getNodeWidth(*itnright);
			      if(WS > 0) //WS found
				{
				  if(WS > xDiff)
				    WS=xDiff;
				  xDiff-=WS;
				  for(itntemp=itn+1;itntemp!=itnright+1;
				      ++itntemp)
				    {
				      _placement[*itntemp].x+=WS;
				    }
				}
			      
			    }
			  else //search left for WS
			    {
			      itntemp=itnleft;
			      --itnleft;
			      WS=_placement[*itntemp].x -
				_placement[*itnleft].x -
				_hgWDims->getNodeWidth(*itnleft);
			      if(WS > 0) //WS found
				{
				  if(WS > xDiff)
				    WS=xDiff;
				  xDiff-=WS;
				  for(itntemp=itn;itntemp!=itnleft;--itntemp)
				    {
				      _placement[*itntemp].x-=WS;
				    }
				}
			    }
			}

			  if(xDiff==0)
			    numOverlaps--; //1 overlap removed
		      //now plot the removed overlap areas
		      sprintf(plotFile,"%s%d","overlaps_rem",totalOverlaps);
		      //to plot the removed overlaps uncomment out the following line
		      //saveAsPlot("plotNodesWNames",xMin,xMax,yMin,yMax,plotFile);
		      
		    }
		}


	      //check for overlaps for >1 rowHeight cells in upper rows
	      if((nodeHeight > rowHeight)&&(itc!=_coreRows.end()-1))
		{
		  itcNext = itc+1;
		  for(itsNext=itcNext->subRowsBegin();
		      itsNext!=itcNext->subRowsEnd();++itsNext)
		    {
		      for(itnNext=itsNext->cellIdsBegin();
			  itnNext!=itsNext->cellIdsEnd();++itnNext)
			{
			  //check for overlaps in the upper adjacent  row
			  if(itnNext!=itsNext->cellIdsEnd())
			    {
			      if((_placement[*itnNext].x>=_placement[*itn].x&&
				 _placement[*itnNext].x<=_placement[*itn].x+
				  _hgWDims->getNodeWidth(*itn)) ||
				 (_placement[*itnNext].x+
				  _hgWDims->getNodeWidth(*itnNext)>=
				  _placement[*itn].x&&
				  _placement[*itnNext].x+
				  _hgWDims->getNodeWidth(*itnNext)<=
				  _placement[*itn].x+
				  _hgWDims->getNodeWidth(*itn)))
				{
				  //overlap found, now deal with it
				  numOverlaps++;
				  totalOverlaps++;
				  
				  //first plot the overlap area
				  sprintf(plotFile,"%s%d","overlaps",
					  totalOverlaps);
/*				  double xMin=_placement[*itn].x -
				    5*_hgWDims->getNodeWidth(*itn);
				  double xMax=_placement[*itn].x +
				    5*_hgWDims->getNodeWidth(*itn);
				  double yMin=_placement[*itn].y -
				    2*_hgWDims->getNodeWidth(*itn);
				  double yMax=_placement[*itn].y +
				    2*_hgWDims->getNodeWidth(*itn);
*/				  //to plot the overlaps uncomment out the folowing line
				  //saveAsPlot("plotNodes",xMin,xMax,yMin,yMax,plotFile);
				  
				  if(_placement[*itnNext].x>=
				     _placement[*itn].x&&
				     _placement[*itnNext].x<=
				     _placement[*itn].x+_hgWDims->getNodeWidth(*itn))
				    xDiff= _placement[*itn].x + 
				      _hgWDims->getNodeWidth(*itn)-
				      _placement[*itnNext].x;

				  else
				    xDiff= _placement[*itnNext].x + 
				     _hgWDims->getNodeWidth(*itnNext)-
				      _placement[*itn].x;
				  
				  
				  itnleft=itnNext;
				  itnright=itnNext;
				  leftRight=false;
				  
				  while(xDiff>0 && 
					(itnleft!=(itsNext->cellIdsBegin()) || 
					 itnright!=(itsNext->cellIdsEnd()-1)))
				    {
				      // chose the direction of search
				      if(leftRight==false && 
					 itnright!=itsNext->cellIdsEnd()-1)
					leftRight=true;
				      else if(leftRight==true && 
					      itnleft!=itsNext->cellIdsBegin())
					leftRight=false;
				      
				      if(leftRight==true)//search right for WS
					{
					  itntemp=itnright+1;
					  WS=_placement[*itntemp].x -
					    _placement[*itnright].x -
					    _hgWDims->getNodeWidth(*itnright);
					  if(WS > 0 && 
					     (_placement[*itntemp].x>
					      (_placement[*itnNext].x+
					       _hgWDims->getNodeWidth(*itnNext)))) //WS found
					    {
					      if(WS > xDiff)
						WS=xDiff;
					      for(itntemp=itnNext;
						  itntemp!=itnright+1;++itntemp)
						{
						  _placement[*itntemp].x+=WS;
						}
					    }
					  ++itnright;
					}
				      else //search left for WS
					{
					  itntemp=itnleft;
					  --itnleft;
					  WS=_placement[*itntemp].x -
					    _placement[*itnleft].x -
					    _hgWDims->getNodeWidth(*itnleft);
					  if(WS > 0 && 
					     ((_placement[*itnleft].x+
					       _hgWDims->getNodeWidth(*itnleft))<
					      _placement[*itnNext].x)) //WS found
					    {
					      if(WS > xDiff)
						WS=xDiff;
					      for(itntemp=itnNext;
						  itntemp!=itnleft;--itntemp)
						{
						  _placement[*itntemp].x-=WS;
						}
					    }
					}
				      if(_placement[*itnNext].x>=
					 _placement[*itn].x&&
					 _placement[*itnNext].x<=
					 _placement[*itn].x+
					 _hgWDims->getNodeWidth(*itn))
					xDiff= _placement[*itn].x + 
					  _hgWDims->getNodeWidth(*itn)-
					  _placement[*itnNext].x;
				      else
					xDiff= _placement[*itnNext].x + 
					  _hgWDims->getNodeWidth(*itnNext)-
					  _placement[*itn].x;
				      
				    }
				  if(xDiff==0)
				    numOverlaps--;
				  //now plot the removed overlap areas
				  sprintf(plotFile,"%s%d","overlaps_rem",
					  totalOverlaps);
				  //to plot the removed overlaps uncomment out the following line
				  //saveAsPlot("plotNodes",xMin,xMax,yMin,yMax,plotFile);
				}
			    }
			}
		    }
		}
	    }
	}
    }
  tm.stop();
  double HPWLafter=evalHPWL();
  double HPWLwPins=evalHPWL(true);
  if (_params.verb.forMajStats) 
  {
    cout<<" Removing overlaps took " << tm << endl;
    cout<<"  Discovered " << totalOverlaps << " overlaps "<<endl;
    cout<<"  Overlaps remaining: "<<numOverlaps<<endl;
    cout<<"  HPWL after  overlap removal is "<<HPWLafter<<endl;
    cout<<"  HPWL w pins                 is "<<HPWLwPins<<endl;
    cout<<"  % change in HPWL is "<<(HPWLafter-initHPWL)*100/HPWLafter<<endl;
  }
}




//added by sadya to remove overlaps in each subrow
void RBPlacement::remOverlapsVert(void)
{
  if (_params.verb.forActions) 
    cout << " Running overlap removal with Vertical Moves..." << endl;
  double initHPWL=evalHPWL();
  if (_params.verb.forMajStats)
    cout << "  HPWL before overlap remover is "<<initHPWL<<endl;
  Timer tm;
  
  itRBPCoreRow itc;
  itRBPSubRow its;
  itRBPCellIds itn;
  itRBPCellIds itntemp;
  itRBPCoreRow itcNext;   //used to scan immediate next row
  itRBPSubRow itsNext;    //used to scan immediate next sub row 
  itRBPCellIds itnNext;   //used to scan immediate next sub row cells
  itRBPCellIds itnright;   //used to scan immediate next sub row cells
  itRBPCellIds itnNextLeft; //used to scan 
  itRBPCellIds itnNextRight; //used to scan 
  
  RBPSubRow tempSR;
  
  bool leftRight=false;   //false means move left else move right
  double WS=0;
  unsigned numOverlaps=0;
  unsigned totalOverlaps=0;
  char plotFile[20];
  double rowHeight=0;
  double xDiff=0;
  //for each core row, for each subrow scan left to right if overlap found then take action
  for(itc=_coreRows.begin();itc!=_coreRows.end();++itc)
    {
      rowHeight=itc->getHeight();
      for(its=itc->subRowsBegin();its!=itc->subRowsEnd();++its)
	{
	  for(itn=its->cellIdsBegin();itn!=its->cellIdsEnd();++itn)
	    {
	      //check for overlaps in the same row
	      xDiff=1;
	      if(itn!=its->cellIdsEnd()-1)
		{
		  itntemp=itn+1;
		  if(_placement[*itntemp].x < 
		     _placement[*itn].x+_hgWDims->getNodeWidth(*itn))
		    {
		      //overlap found, now deal with it
		      numOverlaps++;
		      totalOverlaps++;

		      //first plot the overlap area
		      sprintf(plotFile,"%s%d","overlaps",totalOverlaps);
/*		      double xMin=_placement[*itn].x -
			5*_hgWDims->getNodeWidth(*itn);
		      double xMax=_placement[*itn].x +
			5*_hgWDims->getNodeWidth(*itn);
		      double yMin=_placement[*itn].y -
			2*_hgWDims->getNodeWidth(*itn);
		      double yMax=_placement[*itn].y +
			2*_hgWDims->getNodeWidth(*itn);
*/		      //to plot the overlaps uncomment out the folowing line
		      //saveAsPlot("plotNodesWNames",xMin,xMax,yMin,yMax,plotFile);

		      xDiff= _placement[*itn].x + 
			_hgWDims->getNodeWidth(*itn)-_placement[*itntemp].x;
		      itnright = itntemp;

		      //check for WS in upper rows
		      if(itc!=_coreRows.end()-1)
			{
			  itcNext = itc+1;
			  for(itsNext=itcNext->subRowsBegin();
			      itsNext!=itcNext->subRowsEnd();++itsNext)
			    {
			      for(itnNext=itsNext->cellIdsBegin();
				  itnNext!=itsNext->cellIdsEnd()-1;++itnNext)
				{
				  //check for WS in the upper adjacent  row
				  if((_placement[*itnNext].x <=
				      _placement[*itn].x) &&
				     (_placement[*(itnNext+1)].x >=
				      _placement[*itn].x))
				    break;   //closest cell found in upper
				             //row
				}
			      itnNextLeft = itnNext;
			      itnNextRight = itnNext;
			      
			      while(itnNextRight!=itsNext->cellIdsEnd()-1 ||
				    itnNextLeft!=itsNext->cellIdsBegin())
				{
				  if(leftRight==false && 
				     itnNextRight!=itsNext->cellIdsEnd()-1)
				    leftRight=true;
				  else if(leftRight==true && 
					  itnNextLeft!=itsNext->cellIdsBegin())
				    leftRight=false;
				  
				  if(leftRight == true)  //search right
				    {
				      itntemp=itnNextRight+1;
				      WS=_placement[*itntemp].x -
					_placement[*itnNextRight].x -
					_hgWDims->getNodeWidth(*itnNextRight);
				      if(WS > 0)   //white space found
					{
					  if(WS >=_hgWDims->getNodeWidth(*itn))
					    {
					      tempSR = *its;
					      tempSR.removeCell(*itn);
					      _placement[*itn].x = _placement[*itnNextRight].x;
					      _placement[*itn].y = itsNext->getYCoord();
					      tempSR = *itsNext;
					      tempSR.addCell(*itn);
					      --numOverlaps;
					      break;
					    }
					  else if(WS >= _hgWDims->getNodeWidth(*itnright))
					    {
					      tempSR = *its;
					      tempSR.removeCell(*itnright);
					      _placement[*itnright].x = _placement[*itnNextRight].x;
					      _placement[*itnright].y = itsNext->getYCoord();
					      tempSR = *itsNext;
					      tempSR.addCell(*itnright);
					      --numOverlaps;
					      break;
					    }
					  else
					    {
					      
					    }
					}
				      ++itnNextRight;
				    }
				  else //search left for WS
				    {
				      itntemp=itnNextLeft-1;
				      WS = _placement[*itnNextLeft].x -
					_placement[*itntemp].x -
					_hgWDims->getNodeWidth(*itntemp);
				      if(WS > 0)   //white space found
					{
					  if(WS >=_hgWDims->getNodeWidth(*itn))
					    {
					      tempSR = *its;
					      tempSR.removeCell(*itn);
					      _placement[*itn].x=_placement[*itntemp].x;
					      _placement[*itn].y=itsNext->getYCoord();
					      tempSR = *itsNext;
					      tempSR.addCell(*itn);
					      --numOverlaps;
					      break;
					    }
					  else if(WS >= _hgWDims->getNodeWidth(*itnright))
					    {
					      tempSR = *its;
					      tempSR.removeCell(*itnright);
					      _placement[*itnright].x = _placement[*itntemp].x;
					      _placement[*itnright].y=itsNext->getYCoord();
					      tempSR = *itsNext;
					      tempSR.addCell(*itnright);
					      --numOverlaps;
					      break;
					    }
					}
				      itnNextLeft--;
				    }
				}

/*				  double xMin=_placement[*itn].x -
				    5*_hgWDims->getNodeWidth(*itn);
				  double xMax=_placement[*itn].x +
				    5*_hgWDims->getNodeWidth(*itn);
				  double yMin=_placement[*itn].y -
				    2*_hgWDims->getNodeWidth(*itn);
				  double yMax=_placement[*itn].y +
				    2*_hgWDims->getNodeWidth(*itn);
*/				  //to plot the overlaps uncomment out the folowing line
				  //saveAsPlot("plotNodes",xMin,xMax,yMin,yMax,plotFile);
			    }
			}
		    }
		}
	    }
	}
    }
  tm.stop();
  double HPWLafter=evalHPWL();
  double HPWLwPins=evalHPWL(true);
  if (_params.verb.forMajStats) 
    {
      cout<<" Removing overlaps took " << tm << endl;
      cout<<"  Discovered " << totalOverlaps << " overlaps "<<endl;
      cout<<"  Overlaps remaining: "<<numOverlaps<<endl;
      cout<<"  HPWL after  overlap removal is "<<HPWLafter<<endl;
      cout<<"  HPWL w pins                 is "<<HPWLwPins<<endl;
      cout<<"  % change in HPWL is "<<(HPWLafter-initHPWL)*100/HPWLafter<<endl;
    }
}
